require "class"
require "object"

function angle_diff(a1, a2)
    local a = (a2 - a1)
    
    if (a > math.pi) then
        a = a - 2*math.pi
    elseif (a < -math.pi) then
        a = a + 2*math.pi
    end

    return a
end

class "Guard" (Object) {
    frames = {
        love.graphics.newImage("graphics/guard/guardWalk1.png"),
        love.graphics.newImage("graphics/guard/guardWalk2.png"),
        love.graphics.newImage("graphics/guard/guardWalk3.png"),
        love.graphics.newImage("graphics/guard/guardWalk4.png"),
        love.graphics.newImage("graphics/guard/guardWalk5.png"),
        love.graphics.newImage("graphics/guard/guardWalk6.png"),
    },
    torch = love.graphics.newImage("graphics/torch.png"),
    alertg = love.graphics.newImage("graphics/alert.png"),
    huh = love.graphics.newImage("graphics/huh.png"),
    speed = 132,
    layer = 3,
    anim_speed = 0.5,
}

function Guard:__init__(x, y, a)
    Object.__init__(self, x, y, a)

    self.tx, self.ty = x, y
    self.px, self.py = self.tx, self.ty

    self.shape = shapes.newCircleShape(x, y, 11)
    self.shape.info = {type = "guard", object = self}

    self.anim = 0

    self.alerts = {
    }

    self.alert_timeout = 0
    self.huhtimer = 0
    self.oldmantick = 0
end

function Guard:update(dt)
    Object.update(self, dt)

    local clear = {}
    local best_v, best_alert = 0

    print(#self.alerts)

    for alert in pairs(self.alerts) do
        if alert.alert > best_v then
            best_v = alert.alert
            best_alert = alert
        end

        alert.alert = alert.alert - 0.1*dt

        if alert.alert <= 0 then
            clear[alert] = true
        end
    end

    for alert in pairs(clear) do
        self.alerts[alert] = nil
    end

    if best_alert then
        self.tx = best_alert.x
        self.ty = best_alert.y
    end

    local dx, dy = self.tx - self.x, self.ty - self.y
    local d = math.sqrt(dx*dx + dy*dy)

    if d > 32 then
        local dx2, dy2 = self.px - self.x, self.py - self.y

        self.vx, self.vy = self.speed * math.cos(self.a), self.speed * math.sin(self.a)
    else
        if best_alert then self.alerts[best_alert] = nil end

        self.vx, self.vy = 0, 0
        self.tx, self.ty = math.random(11, 1024-11), math.random(11, 768-11)
    end

    self.anim = (self.anim + self.anim_speed * dt) % 1

    self.shape:moveTo(self.x, self.y)

    local dx, dy = player.x - self.x, player.y - self.y
    local d = math.sqrt(dx*dx + dy*dy)

    local ta = math.atan2(dy, dx)
    local da = angle_diff(self.a, ta)

    if math.abs(da) < math.pi*2 / 6 and d < 128 then
        self:alert(player.x, player.y, 1)
        self.alert_timeout = 1

        if self.oldmantick <= 0 then
            self.oldmantick = 5
            sounds.oldman[math.random(1,3)]:play()
        end
    end

    if d < 32 then
        lose()
    end

    self.alert_timeout = math.max(0, self.alert_timeout - dt)
    self.huhtimer = math.max(0, self.huhtimer - dt)
    self.oldmantick = math.max(0, self.oldmantick - dt)

    local x1, y1 = math.floor(self.x/32+1), math.floor(self.y/32+1)
    local x2, y2 = math.floor(self.tx/32+1), math.floor(self.ty/32+1)

    path = pather:getPath(x1, y1, x2, y2)

    if path and #path >= 2 then
        self.px, self.py = (path[2].x*32-5), (path[2].y*32-5)

        local x2, y2 = (path[1].x*32-5), (path[1].y*32-5)
        local ta = math.atan2(self.py - self.y, self.px - self.x)

        local da = angle_diff(self.a, ta)

        self.a = self.a + math.min(dt*5, da)
    elseif path then
        local dx, dy = self.tx - self.x, self.ty - self.y
        local ta = math.atan2(dy, dx)
        local da = angle_diff(self.a, ta)

        self.a = self.a + math.min(dt*5, da)
    else
        self.tx, self.ty = math.random(11, 1024-11), math.random(11, 768-11)
        self.px, self.py = self.tx, self.ty
    end
end

function Guard:draw()
    local frame = math.floor(self.anim * (#self.frames - 1)) + 1

    love.graphics.setColor(255, 255, 255)
    love.graphics.setBlendMode("alpha")
    love.graphics.draw(self.frames[frame], self.x, self.y, self.a, 1, 1, 13, 21)
    love.graphics.setBlendMode("additive")
    love.graphics.draw(self.torch, self.x, self.y, self.a, 1, 1, 13, 21)

    love.graphics.setColor(255, 255, 255, 255 * self.alert_timeout)
    love.graphics.draw(self.alertg, self.x, self.y - 10, 0, 1, 1, 12, 31) 

    love.graphics.setColor(255, 255, 255, 255 * self.huhtimer)
    love.graphics.draw(self.huh, self.x, self.y - 10, 0, 1, 1, 12, 31) 
end

function Guard:block(mtv_x, mtv_y)
    self.x, self.y = self.x + mtv_x, self.y + mtv_y
end

function Guard:alert(x, y, alert)
    local alert2 = {
        x = x, y = y,
        alert = alert,
    }

    self.alerts[alert2] = true

    if alert > 0.15 and alert < 1 then
        self.huhtimer = 1
    end
end
