Jumper = require "jumper"

require "class"
require "world"

require "level"
require "player"
require "guard"

function lose()
    lost = true

    lose_shader:send("x", player.x)
    lose_shader:send("y", 768-player.y)
    lose_shader:send("r", 768)

    love.audio.stop()
    love.graphics.setPixelEffect(lose_shader)
    sounds.lose:play()
end

function win()
    won = true

    lose_shader:send("x", player.x)
    lose_shader:send("y", 768-player.y)
    lose_shader:send("r", 768)

    love.audio.stop()
    love.graphics.setPixelEffect(lose_shader)
    sounds.win:play()
end

function love.load()
    started = false
    lost = false
    won = false
    busted = false
    restart = false
    instruct = false
    losetime = 4
    titlefade = 1
    splashfade = 1
    splashwait = 2

    scoreflash = 0

    title = love.graphics.newImage("graphics/start.png")
    instructions = love.graphics.newImage("graphics/instructions.png")
    lostscreen = love.graphics.newImage("graphics/lose.png")
    wonscreen = love.graphics.newImage("graphics/win.png")
    splash = love.graphics.newImage("graphics/splash.png")


    lose_shader = love.graphics.newPixelEffect [[
        extern number x, y, r;

        vec4 effect(vec4 colour, Image image, vec2 tex, vec2 screen)
        {
            vec2 center = vec2(x, y);

            if (distance(center, screen) < r) {
                return Texel(image, tex) * colour;
            } else {
                return vec4(0, 0, 0, 1);
            }
        }
    ]]

    world = World()

    player = Player(512, 768/2)
    guard = Guard(300, 100)

    world:add(player)
    world:add(guard)

    sounds = {
        music = love.audio.newSource("sounds/Hide and Sneak.ogg"),
        land = love.audio.newSource("sounds/land.ogg", "static"),
        throw = love.audio.newSource("sounds/throw.ogg", "static"),
        pickup = love.audio.newSource("sounds/pickup.ogg", "static"),
        footsteps = love.audio.newSource("sounds/footsteps.ogg", "static"),
        lose = love.audio.newSource("sounds/lose.ogg", "static"),
        win = love.audio.newSource("sounds/win.ogg", "static"),

        oldman = {
            love.audio.newSource("sounds/1.ogg", "static"),
            love.audio.newSource("sounds/2.ogg", "static"),
            love.audio.newSource("sounds/3.ogg", "static")
        },
    }

    for _, sound in pairs(sounds.oldman) do
        sound:setVolume(1.5)
    end

    --sounds.pickup:setVolume(1)

    sounds.music:setLooping(true)
    sounds.music:play()
    sounds.footsteps:setLooping(true)

    function collide(dt, shape_one, shape_two, mtv_x, mtv_y)
        local player, projectile, obstacle, person

        if shape_one.info.type == "player" then
            player = shape_one.info.object
            person = shape_one.info.object
        elseif shape_two.info.type == "player" then
            player = shape_two.info.object
            person = shape_two.info.object
        end

        if shape_one.info.type == "guard" then
            person = shape_one.info.object
        elseif shape_two.info.type == "guard" then
            person = shape_two.info.object
        end

        if shape_one.info.type == "projectile" then
            projectile = shape_one.info.object
        elseif shape_two.info.type == "projectile" then
            projectile = shape_two.info.object
        end

        if shape_one.info.type == "obstacle" then
            obstacle = shape_one.info.object
        elseif shape_two.info.type == "obstacle" then
            obstacle = shape_two.info.object
        end

        if player and projectile then
            player:grab(projectile)
        end

        if person and obstacle then
            person:block(mtv_x, mtv_y)
        end
    end

    world.collider:setCallbacks(collide, nil)

    level = Level()

    score_gem = love.graphics.newImage("graphics/bigGem.png")
    score_font = love.graphics.newFont("graphics/score.otf", 30)
    final_font = love.graphics.newFont("graphics/score.otf", 50)

    lights = love.graphics.newImage("graphics/lights.png")
    overlay = love.graphics.newImage("graphics/overlay.png")

    motes = {
        love.graphics.newImage("graphics/motes1.png"),
        love.graphics.newImage("graphics/motes2.png"),
        love.graphics.newImage("graphics/motes3.png"),
        love.graphics.newImage("graphics/motes4.png")
    }
    motetimer = 0

    door = love.graphics.newImage("graphics/door.png")

    grid = {}

    local yi = 1
    local circle = shapes.newCircleShape(0, 0, 11)

    for y = 0,768,32 do
        grid[yi] = {}

        local xi = 1

        for x = 0,1024,32 do
            local passable = true

            circle:moveTo(x + 16, y + 16)

            for shape in pairs(world.collider:shapesInRange(x+5,y+5,x+27,y+27)) do
                if shape.info.type == "obstacle"
                and shape:collidesWith(circle) then
                    passable = false
                    break
                end
            end

            grid[yi][xi] = passable

            xi = xi + 1
        end

        yi = yi + 1
    end

    pather = Jumper(grid, true, true)
    world.collider:update(0)
end

function love.update(dt)
    splashwait = math.max(splashwait - dt, 0)

    if splashwait > 0 then return end

    splashfade = math.max(splashfade - 3*dt, 0)

    if splashfade > 0 then return end

    if started and not won and not lost then
        world:update(dt)
        titlefade = math.max(titlefade - dt, 0)
    end

    scoreflash = math.max(scoreflash - dt, 0)

    if lost then
        sounds.music:stop()
        losetime = math.max(0, losetime - dt)
        lose_shader:send("r", 64 + 512 * math.max(0, losetime/2 - 0.5))

        lose_shader:send("x", player.x)
        lose_shader:send("y", 768-player.y)

        if losetime <= 0 then
            busted = true
        end
    end

    if won then
        losetime = math.max(0, losetime - dt)
        lose_shader:send("r", 64 + 512 * math.max(0, losetime/2 - 0.5))

        lose_shader:send("x", player.x)
        lose_shader:send("y", 768-player.y)
    end

    if restart then
        love.audio.stop()
        love.load()
        love.graphics.reset()
    end

    motetimer = (motetimer + 0.5*dt) % 1
end

function love.draw()
    ----[[
    if started then
        level:draw()
        world:draw()

        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.setBlendMode("alpha")

        local score = player.swag * 500

        love.graphics.setColor(0, 0, 0, 64)
        love.graphics.rectangle("fill", 0, 675, 1024, 128)
        love.graphics.setColor(125+128*scoreflash, 224-128*scoreflash, 245-128*scoreflash, 255)
        love.graphics.setFont(score_font)
        love.graphics.print(" x" .. player.swag .. "    £" .. score, 70, 768 - 65)
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(score_gem, 17, 685)

        love.graphics.setBlendMode("additive")
        love.graphics.draw(lights, 0, -8)

        local frame = math.floor(motetimer * (#motes-1)) + 1
        love.graphics.draw(motes[frame], 120, 40, 0, 0.8, 0.8, 50, 20)
        love.graphics.draw(motes[frame], 330, 40, 0, 0.8, 0.8, 50, 20)
        love.graphics.draw(motes[frame], 515, 40, 0, 0.8, 0.8, 50, 20)
        love.graphics.draw(motes[frame], 700, 40, 0, 0.8, 0.8, 50, 20)
        love.graphics.draw(motes[frame], 880, 40, 0, 0.8, 0.8, 50, 20)
    end

    love.graphics.setBlendMode("alpha")
    love.graphics.setColor(255, 255, 255, 255 * titlefade)
    love.graphics.draw(title)

    love.graphics.setBlendMode("alpha")
    love.graphics.setColor(255, 255, 255, 255 * splashfade)
    love.graphics.draw(splash)

    if instruct then 
        love.graphics.setBlendMode("alpha")
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(instructions)
    end


    love.graphics.setColor(255, 255, 255, 255)
    if lost and losetime <= 0 then
        love.graphics.draw(lostscreen)
        love.graphics.setPixelEffect()
    end

    if won and losetime <= 0 then
        love.graphics.setBlendMode("alpha")
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(wonscreen)
        love.graphics.setPixelEffect()

        local score = player.swag * 500
        love.graphics.setColor(125, 224, 245, 255)
        love.graphics.setFont(final_font)
        love.graphics.printf("£" .. score, 0, 360, 1024, "center")
        love.graphics.setColor(255, 255, 255, 255)
    end
    --]]

    --[[
    love.graphics.setBlendMode("alpha")
    if path then
        local px, py = path[1].x, path[1].y

        for index, node in ipairs(path) do  
            local cx, cy = node.x, node.y

            love.graphics.setColor(255, 255, 255, 255)
            love.graphics.line(px*32-5, py*32-5, cx*32-5, cy*32-5)

            px, py = cx, cy
        end
    end
    --]]

    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.setBlendMode("alpha")
    love.graphics.draw(overlay)

    --love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)
end

function love.mousepressed(x, y, button)
    print(started, instruct)
    if started then
        if button == "r" and not lost then
            player:fire(x, y)
            scoreflash = 1
        end
    elseif not instruct then
        print("hello")
        if love.mouse.getY() >= 510 and love.mouse.getY() <= 555 then
            print("started")
            started = true
        elseif love.mouse.getY() >= 555 and love.mouse.getY() <= 610 then
            instruct = true
        end
    elseif instruct then
        if love.mouse.getY() >= 690 then instruct = false end
    end

    if lost or won and losetime <= 0 then
        restart = true
    end
end
