shapes = require "collider.shapes"

require "class"
require "object"

require "projectile"

class "Player" (Object) {
    frames = {
        love.graphics.newImage("graphics/player/walk1.png"),
        love.graphics.newImage("graphics/player/walk2.png"),
        love.graphics.newImage("graphics/player/walk3.png"),
        love.graphics.newImage("graphics/player/walk4.png"),
        love.graphics.newImage("graphics/player/walk5.png"),
        love.graphics.newImage("graphics/player/walk6.png"),
    },
    idles = {
        love.graphics.newImage("graphics/player/idle1.png"),
        love.graphics.newImage("graphics/player/idle2.png"),
    },

    shadows = {
        love.graphics.newImage("graphics/shadow1.png"),
        love.graphics.newImage("graphics/shadow2.png"),
    },

    speed = 128,
    anim_speed = 0.5,
    layer = 3,
}

function Player:__init__(x, y, a)
    Object.__init__(self, x, y, a)

    self.shape = shapes.newCircleShape(x, y, 11)
    self.shape.info = {type = "player", object = self}

    self.swag = 0
    self.anim = 0
    self.walk_timer = 0
    self.idle = false
end

function Player:update(dt)
    Object.update(self, dt)

    print("update player")

    local mx, my = love.mouse.getPosition()
    self.a = math.atan2(my - self.y, mx - self.x)

    self.anim = (self.anim + self.anim_speed * dt) % 1

    self.shape:moveTo(self.x, self.y)

    if love.mouse.isDown("l") and not lost then
        self.vx = math.cos(self.a) * self.speed
        self.vy = math.sin(self.a) * self.speed
    else
        self.vx, self.vy = 0, 0
    end

    self:walk(dt)

    local dx, dy = self.x - 30, math.abs(self.y - 768/2)
    if dx < 30 and dy < 103 then
        win()
    end
end

function Player:draw()
    local frame = math.floor(self.anim * (#self.shadows-1)) + 1
    love.graphics.draw(self.shadows[frame], self.x, self.y, 0, 1, 1, 8, 0)

    local frame = math.floor(self.anim * (#self.frames-1)) + 1

    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.setBlendMode("alpha")

    if self.idle then
        local frame = math.ceil(self.anim * #self.idles)
        love.graphics.draw(self.idles[frame], self.x, self.y, self.a, 1, 1, 20, 30)
    else
        love.graphics.draw(self.frames[frame], self.x, self.y, self.a, 1, 1, 20, 30)
    end
end

function Player:grab(projectile)
    if projectile.motion then return end

    self.swag = self.swag + 1
    sounds.pickup:stop()
    sounds.pickup:play()
    world:remove(projectile)
end

function Player:walk(dt)
    if self.vx*self.vx + self.vy*self.vy < 5*5 then
        self.idle = true
        sounds.footsteps:stop()
        return
    else
        sounds.footsteps:play()
    end

    self.idle = false

    self.walk_timer = self.walk_timer + dt

    if self.walk_timer > 0.5 then
        self.walk_timer = self.walk_timer - 0.5
        world:add(Sound(self.x, self.y, 0.1))
    end
end

function Player:block(mtv_x, mtv_y)
    self.x, self.y = self.x + mtv_x, self.y + mtv_y
end

function Player:fire(x, y)
    if self.swag <= 0 then return end

    self.swag = self.swag - 1

    local gx = self.x + 8 * math.cos(self.a)
    local gy = self.y + 8 * math.sin(self.a)

    --local vx, vy = 512 * math.cos(self.a), 512 * math.sin(self.a)

    world:add(Projectile(gx, gy, x, y))

    sounds.throw:stop()
    sounds.throw:play()
end
