Collider = require "collider"

require "class"

class "World" {}

function World:__init__(gridsize)
    self.objects = {}
    self.changes = {}

    self.collider = Collider(gridsize or 32)
end

function World:add(object)
    self.changes[object] = true
    if object.shape then 
        self.collider:addShape(object.shape) 
        if object.shape.info.type == "obstacle"
        or object.shape.info.type == "projectile" then
            world.collider:setPassive(object.shape)
        end
    end
end

function World:remove(object)
    self.changes[object] = false
    if object.shape then self.collider:remove(object.shape) end
end

function World:update(dt)
    for object, added in pairs(self.changes) do
        if added then
            self.objects[object] = true
        else
            self.objects[object] = nil
        end
    end

    self.changes = {}

    for object in pairs(self.objects) do
        object:update(dt)
    end

    self.collider:update(dt)
end

function World:draw()
    local max_layer = 1

    for object in pairs(self.objects) do
        max_layer = math.max(max_layer, object.layer)
    end

    local layers = {}

    for i=1,max_layer do
        layers[i] = {}
    end

    for object in pairs(self.objects) do
        layers[object.layer][object] = true
    end

    for layer, objects in ipairs(layers) do
        for object in pairs(objects) do
            object:draw()
        end
    end

    love.graphics.setBlendMode("alpha")
    love.graphics.setColor(128, 192, 192, 255)
    --self.collider._hash:draw("line", false)
end
